package com.xhkjedu.sexam.service.paper;

import com.xhkjedu.sexam.mapper.exam.ESubjectMapper;
import com.xhkjedu.sexam.mapper.paper.*;
import com.xhkjedu.sexam.model.exam.ESubject;
import com.xhkjedu.sexam.model.paper.*;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description 附件试卷
 * @Author WN
 * Date 2022/7/18 14:05
 **/
@Service
public class EPaperFjtypeService {
    @Resource
    private EPaperFjtypeMapper ePaperFjtypeMapper;
    @Resource
    private EPaperFjtypeQuestionMapper ePaperFjtypeQuestionMapper;
    @Resource
    private ESubjectMapper eSubjectMapper;
    @Resource
    private EPaperMapper ePaperMapper;
    @Resource
    private EPaperFileMapper ePaperFileMapper;
    @Resource
    private EPaperAnalyzeMapper ePaperAnalyzeMapper;


    public void saveFjPaper(EPaper paper){
        int epid = 0;

        //获取考试科目信息
        ESubject eSubject = eSubjectMapper.selectByPrimaryKey(paper.getEsid());

        paper.setPtype(2);//试卷类型1题库2答题卡3仅答题卡
        paper.setExamid(eSubject.getExamid());
        paper.setSubjectid(eSubject.getSubjectid());
        int timestamp = N_Utils.getSecondTimestamp();

        //答题卡种题型和试题
        List<EPaperFjtype> fjtypelist = paper.getQuestionStr();
        //答题卡中所有试题
        List<EPaperFjtypeQuestion> queslist = new ArrayList<>();
        fjtypelist.forEach(q->{
            queslist.addAll(q.getQuestions());
        });
        double pscore = queslist.stream().collect(Collectors.summingDouble(EPaperFjtypeQuestion :: getScore));
        paper.setPnum(queslist.size());
        paper.setPscore(pscore);
        paper.setCreatetime(timestamp);
        paper.setHandletime(timestamp);
        paper.setHandleid(paper.getCreateid());
        ePaperMapper.insertUseGeneratedKeys(paper);
        epid = paper.getId();
        paper.setEpid(epid);

        //保存作业附件
        savePaperFile(paper);
        //保存答题卡信息
        saveFjtypeQuestion(paper,fjtypelist);

        if(paper.getPstate()!=null && paper.getPstate() == 1){
            //如果提交试卷，则生成试卷分析
            savePaperAnalyze(paper,fjtypelist,queslist);
        }
    }

    //保存附件试卷文件
    private void savePaperFile(EPaper paper){
        int fileorder = 1;
        List<EPaperFile> files = paper.getPfiles();
        for(EPaperFile f : files){
            f.setCreateid(paper.getCreateid());
            f.setCreatetime(paper.getCreatetime());
            f.setEpid(paper.getEpid());
            f.setFileorder(fileorder);
            fileorder++;
        }
        ePaperFileMapper.insertList(files);
    }

    //保存题型--题型下试题
    private void saveFjtypeQuestion(EPaper paper,List<EPaperFjtype> fjtypelist){
        int qorder = 1;//答题卡试题排序
        int qtorder = 1;//答题卡题型排序

        List<EPaperFjtypeQuestion> saveques = new ArrayList<>();
        for(EPaperFjtype fjt : fjtypelist){
            List<EPaperFjtypeQuestion> quelist = fjt.getQuestions();//题型下试题
            fjt.setEpfjid(paper.getEpid());
            fjt.setCreateid(paper.getHandleid());
            fjt.setCreatetime(paper.getHandletime());
            fjt.setEpfjorder(qtorder);
            fjt.setEpfjnum(quelist.size());
            double tscore = quelist.stream().collect(Collectors.summingDouble(EPaperFjtypeQuestion :: getScore));
            fjt.setEpfjscore(tscore);
             qtorder ++;
        }
        //批量保存题型信息
        ePaperFjtypeMapper.insertList(fjtypelist);
        for(EPaperFjtype fjt : fjtypelist){
            for(EPaperFjtypeQuestion q : fjt.getQuestions()){
                q.setEpfjid(fjt.getId());
                q.setEpid(paper.getEpid());
                q.setOrder(qorder);
                qorder ++;
                saveques.add(q);
            }
        }
        ePaperFjtypeQuestionMapper.insertList(saveques);
    }

    //保存试卷分析
    public void savePaperAnalyze(EPaper ePaper,List<EPaperFjtype> fjtypelist,List<EPaperFjtypeQuestion> queslist){
        //保存之前先清除之前的试卷分析
        ePaperAnalyzeMapper.deleteByEpid(ePaper.getEpid());
        //总体分析（主观题、客观题）
        //处理题型1单选题2多选题3主观题4判断对错5判断√×6判断TF7完形填空8阅读理解10任务型阅读11综合题12听力
        int znum = 0;
        double zscore = 0;
        int knum = 0;
        double kscore = 0;
        



    }
}
